<?php

namespace FiloBlu\ExtViSualMerchandiser\Model;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Phrase;

/**
 * Class StockStatus
 * @package FiloBlu\ExtViSualMerchandiser\Model
 */
class StockStatus implements StockStatusInterface
{
    /** @var array */
    protected $sku = [];
    /** @var int */
    protected $id;
    /** @var string */
    protected $name;
    /** @var AdapterInterface */
    protected $connection;
    /** @var ResourceConnection */
    private $resourceConnection;

    /**
     * StockStatus constructor.
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(ResourceConnection $resourceConnection)
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @inheritDoc
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @inheritDoc
     */
    public function setId($id)
    {
        $this->id = $id;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @inheritDoc
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @inheritDoc
     * @throws NoSuchEntityException
     */
    public function isInStock($sku)
    {
        if (!isset($this->sku[$sku])) {
            throw new NoSuchEntityException(new Phrase("Sku {$sku} not in collection"));
        }
        return $this->sku[$sku];
    }

    /**
     * @inheritDoc
     */
    public function setIsInStock($sku, $status)
    {
        $this->sku[$sku] = $status;
        return $this;
    }

    /**
     * For MSI.
     *
     * @param string $sku
     * @param int $stockId
     *
     * @return int|string
     */
    private function getReservationQty($sku, $stockId)
    {
        $connection =  $this->getConnection();
        $select = $connection->select()
            ->from($connection->getTableName('inventory_reservation'), ['quantity' => 'SUM(quantity)'])
            ->where('sku = ?', $sku)
            ->where('stock_id = ?', $stockId)
            ->limit(1);

        $reservationQty = $connection->fetchOne($select);
        if ($reservationQty === false) {
            $reservationQty = 0;
        }

        return $reservationQty;
    }

    /**
     * @return AdapterInterface
     */
    private function getConnection()
    {
        if ($this->connection) {
            return $this->connection;
        }

        return ($this->connection = $this->resourceConnection->getConnection());
    }
}
